import requests
import pyrpkg.lookaside
import pycurl
import os
import sys

from pyrpkg.errors import DownloadError

try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse

class CGILookasideCache(pyrpkg.lookaside.CGILookasideCache):
    """A class to interact with a CGI-based lookaside cache"""

    def get_download_url(self, module, filename, hash, outfile, hashtype=None, **kwargs):
        if hashtype is None:
            hashtype = self.hashtype

        self.log.info("Downloading %s from lookaside cache at %s", filename, urlparse(self.download_url).netloc)
        urled_file = filename.replace(' ', '%20')

        subs = {'module': module, 'filename': urled_file,
                'hash': hash, 'hashtype': hashtype}
        subs.update(kwargs)

        module_parts = module.split('/')
        subs['name'] = module_parts[-1]
        ns_index = 1
        for part in reversed(module_parts[:-1]):
            key = 'ns'+str(ns_index)
            subs[key] = part
            ns_index += 1

        try:
            result_url = self.download_url % subs
        except Exception as e:
            raise DownloadError(
                'Could not get download url from %s template. '
                'Original error: %s.' % (self.download_url, str(e)))

        return result_url

    def download(self, module, filename, hash, outfile, hashtype=None, **kwargs):
        if os.path.exists(outfile):
            if self.file_is_valid(outfile, hash, hashtype=hashtype):
                self.log.debug("File %s already downloaded.", outfile)
                return

        download_url = self.get_download_url(
            module, filename, hash, outfile, hashtype, **kwargs)
        self.log.debug("Full url: %s", download_url)

        with open(outfile, 'wb') as f:
            c = pycurl.Curl()
            c.setopt(pycurl.URL, str(download_url))
            c.setopt(pycurl.HTTPHEADER, ['Pragma:'])
            c.setopt(pycurl.NOPROGRESS, False)
            c.setopt(pycurl.PROGRESSFUNCTION, self.print_progress)
            c.setopt(pycurl.OPT_FILETIME, True)
            c.setopt(pycurl.WRITEDATA, f)
            c.setopt(pycurl.LOW_SPEED_LIMIT, 1000)
            c.setopt(pycurl.LOW_SPEED_TIME, 300)
            c.setopt(pycurl.FOLLOWLOCATION, True)

            try:
                c.perform()
                tstamp = c.getinfo(pycurl.INFO_FILETIME)
                status = c.getinfo(pycurl.RESPONSE_CODE)

            except Exception as e:
                raise DownloadError(e)

            finally:
                c.close()

        # Get back a new line, after displaying the download progress
        sys.stdout.write('\n')
        sys.stdout.flush()

        if status != 200:
            self.log.info('Remove downloaded invalid file %s', outfile)
            os.remove(outfile)
            raise DownloadError('Server returned status code %d' % status)

        os.utime(outfile, (tstamp, tstamp))

        if not self.file_is_valid(outfile, hash, hashtype=hashtype):
            raise DownloadError('%s failed checksum' % filename)
